#############################################################################
##
#W  classes.gi  Testing properties/varieties [quandles]
##
#H  @(#)$Id: classes.gi, v 2.1.2 2010/11/16 gap Exp $
##
#Y  Copyright (C)  2004,  G. P. Nagy (University of Szeged, Hungary),
#Y                        P. Vojtechovsky (University of Denver, USA)
##



#############################################################################
##  PROPERTIES OF QUASIGROUPS
##  -------------------------------------------------------------------------

#############################################################################
##
#P  IsCommutative( Q )
##
##  Returns true if <Q> is commutative.

InstallOtherMethod( IsCommutative, "for rack",
    [ IsRack ],
function( Q )
	local x,y;
    for x in Elements(Q) do for y in Elements(Q) do
		if (x*y<>y*x) then return false; fi;
	od;od;
	return true;
end );


#############################################################################
##
#P  IsIdempotent( Q )
##
##  Returns true if the quasigroup <Q> is idempotent, i.e., x*x=x.

InstallMethod( IsIdempotent, "for rack",
    [ IsRack ],
function( Q )
    return ForAll( Q, x -> x*x = x );
end );




#############################################################################
##
#P  IsEntropic( Q )
##
##  Returns true if the quasigroup <Q> is entropic.

InstallMethod( IsEntropic, "for rack",
    [ IsRack ],
function( Q )
    # BETTER ALGORITHM LATER?
    local x, y, z, w;
    for x in Q do for y in Q do for z in Q do for w in Q do
        if not (x*y)*(z*w) = (x*z)*(y*w) then return false; fi;
    od; od; od; od;
    return true;
end );




InstallMethod ( IsIndecomposable, " for affine mesh ",
	[ IsAffineMesh ],
function( M )
	local i,j,l,ge;
	l:=Length(Orbits(M));
	for j in [1..l] do
		ge:=[];
		for i in [1..l] do
			Append(ge,AsList(Elements(Image(HomomorphismsMatrix(M)[i][j]))));
			Add(ge,ConstantsMatrix(M)[i][j]);
		od;
		if SubgroupNC(Orbits(M)[j],ge)<>Orbits(M)[j] then return false;
		fi;
	od;
	return true;
end );



