#############################################################################
##
#W  quandles.gd  Representing, creating and displaying quandles [racks]
##
#H  @(#)$Id: quandles.gd, v 0.1.0 2013/03/04 gap Exp $
##
#Y  Copyright (C)  2013,  P. Jedlicka (CULS Prague)
##

#############################################################################
##  GAP CATEGORIES AND REPRESENTATIONS
##  -------------------------------------------------------------------------

## element of a rack
DeclareCategory( "IsRackElement", IsMultiplicativeElement );
DeclareRepresentation( "IsRackElmRep",
    IsPositionalObjectRep and IsMultiplicativeElement, [1] );

## element of a loop
#DeclareCategory( "IsQuandleElement",
#    IsRackElement and IsMultiplicativeElementWithInverse );
#DeclareRepresentation( "IsQuandleElmRep",
#    IsPositionalObjectRep and IsMultiplicativeElementWithInverse, [1] );

## leftlatin 
DeclareCategory( "IsLeftLatin", IsObject );

## rack
DeclareCategory( "IsRack", IsMagma and IsLeftLatin );

## quandle
DeclareCategory( "IsQuandle", IsRack and IsIdempotent);

## mesh
DeclareCategory( "IsAffineMesh", IsObject);

#############################################################################
##  TESTING MULTIPLICATION TABLES
##  -------------------------------------------------------------------------

DeclareOperation( "IsRackTable", [ IsMatrix ] );
DeclareSynonym( "IsRackCayleyTable", IsRackTable );
DeclareOperation( "IsQuandleTable", [ IsMatrix ] );
DeclareSynonym( "IsQuandleCayleyTable", IsQuandleTable );
#DeclareOperation( "CanonicalCayleyTable", [ IsMatrix ] );
#DeclareOperation( "NormalizedQuasigroupTable", [ IsMatrix ] );

#############################################################################
##  CREATING QUASIGROUPS AND LOOPS MANUALLY
##  -------------------------------------------------------------------------

DeclareAttribute( "CayleyTable", IsRack );
DeclareOperation( "RackByCayleyTable", [ IsMatrix ] );
DeclareOperation( "QuandleByCayleyTable", [ IsMatrix ] );
DeclareOperation( "SetRackElmName", [ IsRack, IsString ] );
DeclareSynonym( "SetQuandleElmName", SetRackElmName );

#############################################################################
##  CREATING QUASIGROUPS AND LOOPS FROM A FILE
##  -------------------------------------------------------------------------
#
#DeclareOperation( "RackFromFile", [ IsString, IsString ] );
#DeclareOperation( "QuandleFromFile", [ IsString, IsString ] );

#############################################################################
##  CREATING QUASIGROUPS AND LOOPS BY SECTIONS
##  -------------------------------------------------------------------------

#DeclareOperation( "CayleyTableByPerms", [ IsPermCollection ] );
#DeclareOperation( "QuasigroupByLeftSection", [ IsPermCollection ] );
#DeclareOperation( "LoopByLeftSection", [ IsPermCollection ] );
#DeclareOperation( "QuasigroupByRightSection", [ IsPermCollection ] );
#DeclareOperation( "LoopByRightSection", [ IsPermCollection ] );
# There are also versions of QuasigroupByRightSection and LoopByRightSection
# for [ IsGroup, IsGroup, IsMultiplicativeElementCollection ]

#############################################################################
##  CONVERSIONS
##  -------------------------------------------------------------------------

#DeclareOperation( "IntoQuasigroup", [ IsMagma ] );
#DeclareOperation( "PrincipalLoopIsotope",
#    [ IsQuasigroup, IsQuasigroupElement, IsQuasigroupElement ] );
#DeclareOperation( "IntoLoop", [ IsMagma ] );
#DeclareOperation( "IntoGroup", [ IsMagma ] );

#############################################################################
## PRODUCTS OF LOOPS
## --------------------------------------------------------------------------

#DirectProduct already declared for groups.

#############################################################################
## OPPOSITE QUASIGROUPS AND LOOPS
## --------------------------------------------------------------------------

#DeclareOperation( "Opposite", [ IsQuasigroup ] );


DeclareOperation("GalkinRepresentation",[IsGroup,IsGroup,IsMapping]);


DeclareOperation("MapNilpotency",[IsMapping]);



#########################################################
##  AFFINE MESHES
##

#DeclareAttribute ( "Orbits", [IsAffineMesh]);

DeclareAttribute ( "ConstantsMatrix", IsAffineMesh);
DeclareAttribute ( "HomomorphismsMatrix", IsAffineMesh);
DeclareAttribute ( "Reductivity", IsAffineMesh);
DeclareOperation ( "AffineMesh" , [IsList,IsList,IsList]);
DeclareOperation ( "AffineMeshNoCheck" , [IsList,IsList,IsList,IsInt]);
DeclareOperation ("QuandleFromAffineMesh",[IsAffineMesh]);

####################################################
#
#  SEARCHING FOR 2-REDUCTIBLES
#
#

DeclareOperation("TwoReductiveConstantMatrix",[IsInt,IsList]);
DeclareOperation("TwoReductiveConstantMatrixInverse",[IsList,IsList]);
DeclareOperation("PermutationsOfOrderedGroupList",[IsList]);
DeclareOperation("LessAffineMeshRow",[IsList,IsList,IsList]);
DeclareOperation("IsSortedConstantMatrix",[IsList,IsInt,IsList]);
DeclareOperation("NumberAffineMeshesZ2Z1",[IsInt,IsInt]);
DeclareOperation("AllTwoReductive",[IsList]);

